package com.web.api.controller.answer;

import com.web.api.constant.CommonConstant;
import com.web.api.exception.ErrorCode;
import com.web.api.exception.SgException;
import com.web.api.model.DataResult;
import com.web.api.model.dto.AnswerDto;
import com.web.api.model.dto.SearchDto;
import com.web.api.service.AnswerService;
import com.web.api.utils.ContextUtils;
import com.web.api.utils.MD5Util;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.CookieStore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.Map;
import java.util.stream.Collectors;


/**
 * @author 王玉鹏
 * @version 1.0
 * @className AnswerController
 * @description TODO
 * @date 2020/7/8 14:05
 */
@RestController
@RequestMapping(CommonConstant.ANSWER_URL_PREFIX + "/api")
@Api(value = "答题管理", tags = "答题管理")
@Slf4j
public class AnswerController {

    @Autowired
    private AnswerService answerService;


    /**
     * @return com.web.api.model.DataResult
     * @Descripton website用户登陆
     * @Param [username, password]
     */
    @RequestMapping(value = "/login", method = RequestMethod.POST)
    @ApiOperation(value = "用户登陆", httpMethod = "POST", notes = "用户登陆")
    public DataResult userLogin(String username, String password, String safeCode, String remember) {
        try {

            return DataResult.ok(answerService.login(username, password, safeCode));
        } catch (Exception e) {
            log.error("登录异常", e);
            return DataResult.fail(e.getMessage());
        }
    }

    @RequestMapping(value = "/user", method = RequestMethod.POST)
    @ApiOperation(value = "用户信息", httpMethod = "POST", notes = "用户信息")
    public DataResult user(String jobNo) {
        try {
            CookieStore cookieStore = ContextUtils.getUserCookies(jobNo);
            return DataResult.ok(answerService.getUserInfo(jobNo, cookieStore));
        } catch (SgException e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getCode(),e.getMessage());
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    @RequestMapping(value = "/anwser", method = RequestMethod.POST)
    @ApiOperation(value = "开始答题", httpMethod = "POST", notes = "答题管理")
    public DataResult answer(AnswerDto answerDto) {
        try {
            if (ObjectUtils.isEmpty(answerDto)) {
                throw new SgException(ErrorCode.PARAM_EMPTY.getCode(), ErrorCode.PARAM_EMPTY.getName());
            }
            answerDto.setBatchFlag(false);
            CookieStore cookieStore = ContextUtils.getUserCookies(answerDto.getJobNo());
            answerService.answer(cookieStore, answerDto);
            return DataResult.ok();
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    @RequestMapping(value = "/getAnwserInfo", method = RequestMethod.POST)
    @ApiOperation(value = "获取当前答题状态", httpMethod = "POST", notes = "获取当前答题状态")
    public DataResult getAnwserInfo(String jobNo) {
        try {
            CookieStore cookieStore = ContextUtils.getUserCookies(jobNo);
            return DataResult.ok(answerService.getAnwserInfo(jobNo, cookieStore));
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    @RequestMapping(value = "/stop", method = RequestMethod.POST)
    @ApiOperation(value = "终止答题", httpMethod = "POST", notes = "终止答题")
    public DataResult stop(String jobNo) {
        try {
            CookieStore cookieStore = ContextUtils.getUserCookies(jobNo);
            answerService.stopAnswer(jobNo, cookieStore);
            return DataResult.ok();
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    @RequestMapping(value = "/user/onlineList", method = RequestMethod.POST)
    @ApiOperation(value = "获取在线用户列表", httpMethod = "POST", notes = "获取在线用户列表")
    public DataResult getOnlineUsers(Integer type) {
        try {
            Map<String, Object> onlineUsers;
            if (type==1){
                onlineUsers = ContextUtils.getOnlineUsers();
            }else {
                onlineUsers = ContextUtils.getBatchUserList();
            }
            return DataResult.ok(onlineUsers.values().stream()
                    .collect(Collectors.toList()));

        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    @RequestMapping(value = "/stopAll", method = RequestMethod.POST)
    @ApiOperation(value = "终止所有答题", httpMethod = "POST", notes = "终止答题")
    public DataResult allStop() {
        try {
             ContextUtils.setStatus(false);
            return DataResult.ok();
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }
    @RequestMapping(value = "/ranking", method = RequestMethod.GET)
    @ApiOperation(value = "排行榜", httpMethod = "GET", notes = "排行榜")
    public DataResult ranking(String jobNo) {
        try {
            CookieStore cookieStore = ContextUtils.getUserCookies(jobNo);
            return DataResult.ok(answerService.ranking(jobNo, cookieStore));
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    @RequestMapping(value = "/exportExcel", method = RequestMethod.POST)
    @ApiOperation(value = "导出excel", httpMethod = "POST", notes = "导出excel")
    public DataResult export(SearchDto searchDto) {
        try {
            return DataResult.ok(answerService.exportExcel(searchDto));
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    @RequestMapping(value = "/logOut", method = RequestMethod.POST)
    @ApiOperation(value = "退出", httpMethod = "POST", notes = "退出")
    public DataResult logOut(String jobNo) {
        try {
            CookieStore cookieStore = ContextUtils.getUserCookies(jobNo);
            answerService.logOut(jobNo, cookieStore);
            return DataResult.ok();
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }
    @RequestMapping(value = "/search", method = RequestMethod.POST)
    @ApiOperation(value = "search", httpMethod = "POST", notes = "search")
    public DataResult search(SearchDto searchDto) {
        try {
            return DataResult.ok(answerService.search(searchDto));
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    @ApiOperation(value = "importExcel", httpMethod = "POST", notes = "importExcel")
    public DataResult importExcel(final MultipartFile file) {
        try {
            return DataResult.ok(answerService.importExcel(file));
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }
    @RequestMapping(value = "/getLimitTime", method = RequestMethod.GET)
    @ApiOperation(value = "getLimitTime", httpMethod = "GET", notes = "getLimitTime")
    public DataResult getLimitTime() {
        try {
            return DataResult.ok(answerService.getLimitTime());
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }
    @RequestMapping(value = "/updateTime", method = RequestMethod.POST)
    @ApiOperation(value = "updateTime", httpMethod = "POST", notes = "updateTime")
    public DataResult updateTime(int time) {
        try {
            answerService.updateTime(time);
            return DataResult.ok("更新成功");
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }
    @RequestMapping(value = "/forbiddenStatus", method = RequestMethod.GET)
    @ApiOperation(value = "getForbiddenStatus", httpMethod = "GET", notes = "getForbiddenStatus")
    public DataResult getForbiddenStatus(String jobNo) {
        try {
            return DataResult.ok(answerService.getForbidden(jobNo));
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }
    @RequestMapping(value = "/forbiddenStatus", method = RequestMethod.POST)
    @ApiOperation(value = "updateForbiddenStatus", httpMethod = "POST", notes = "updateForbiddenStatus")
    public DataResult updateForbiddenStatus(String jobNo,Boolean status) {
        try {
            if (answerService.updateForbiddenStatus(jobNo,status,1)){
                return DataResult.ok("更新成功");
            }else {
                return DataResult.fail("操作失败，请重新尝试");
            }
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }
    @RequestMapping(value = "/resetPassword", method = RequestMethod.POST)
    @ApiOperation(value = "resetPassword", httpMethod = "POST", notes = "updateForbiddenStatus")
    public DataResult resetPassword(String jobNo) {
        try {
            if (answerService.resetPassword(jobNo)){
                return DataResult.ok("重置密码成功");
            }else {
                return DataResult.fail("操作失败，请重新尝试");
            }
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

}
